@@ -112,7 +112,7 @@ class kolab_ldap extends rcube_ldap_generic
                     && (strstr($bind_dn, '%dn') || strstr($base_dn, '%dn') || strstr($groups_base_dn, '%dn'))
                 ) {
                     $search_attribs = ['uid'];
-                    if ($search_bind_attrib = (array) $this->config['search_bind_attrib']) {
+                    if ($search_bind_attrib = (array) ($this->config['search_bind_attrib'] ?? [])) {
                         foreach ($search_bind_attrib as $r => $attr) {
                             $search_attribs[] = $attr;
                             $replaces[$r] = '';
