@@ -866,7 +866,7 @@ class kolab_storage
         if (!self::setup()) {
             rcube::raise_error([
                 'code' => 900, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                'message' => "Failed to init"
+                'message' => "Failed to init",
             ], true, $abortOnError);
             return [];
         }
@@ -887,7 +887,7 @@ class kolab_storage
             if (self::$imap->get_error_code()) {
                 rcube::raise_error([
                     'code' => 900, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                    'message' => "IMAP error: " . self::$imap->get_error_str()
+                    'message' => "IMAP error: " . self::$imap->get_error_str(),
                 ], true, $abortOnError);
             }
 
@@ -902,7 +902,7 @@ class kolab_storage
         if (!is_array($folderdata) || self::$imap->get_error_code()) {
             rcube::raise_error([
                 'code' => 900, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                'message' => "IMAP error: " . self::$imap->get_error_str()
+                'message' => "IMAP error: " . self::$imap->get_error_str(),
             ], true, $abortOnError);
             return [];
         }
@@ -927,10 +927,11 @@ class kolab_storage
         }
 
         // In case of an error, return empty list (?)
+        // @phpstan-ignore-next-line
         if (!is_array($folders) || self::$imap->get_error_code()) {
             rcube::raise_error([
                 'code' => 900, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                'message' => "IMAP error: " . self::$imap->get_error_str()
+                'message' => "IMAP error: " . self::$imap->get_error_str(),
             ], true, $abortOnError);
             return [];
         }
