@@ -97,7 +97,7 @@ class kolab_subscriptions
                 if ($storage->get_error_code()) {
                     rcube::raise_error([
                         'code' => 500, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                        'message' => "IMAP error: " . $storage->get_error_str()
+                        'message' => "IMAP error: " . $storage->get_error_str(),
                     ], true, true);
                 }
 
@@ -122,7 +122,7 @@ class kolab_subscriptions
     /**
      * Getter for folders subscription flag on protected folders
      *
-     * @return string|false false if not set, otherwise the flag.
+     * @return int|false false if not set, otherwise the flag.
      */
     private function get_forced_flag($folder, $devicetype)
     {
@@ -130,13 +130,15 @@ class kolab_subscriptions
 
         foreach ($forcedFolders as $idx => $flag) {
             if (preg_match("|^{$idx}\$|", $folder)) {
-                return $flag;
+                return (int) $flag;
             }
         }
+
         // INBOX should be enabled by default, unless explicitly overridden.
         if (strtolower($folder) == "inbox") {
             return 1;
         }
+
         return false;
     }
 
@@ -176,7 +178,7 @@ class kolab_subscriptions
         if ($type == "mail" && empty($folders)) {
             rcube::raise_error([
                 'code' => 500, 'type' => 'php', 'line' => __LINE__, 'file' => __FILE__,
-                'message' => "Empty mail folder list should not be possible."
+                'message' => "Empty mail folder list should not be possible.",
             ], true, true);
         }
 
