@@ -1,9 +1,10 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -26,42 +27,35 @@
  */
 class rcube_user
 {
+    /** @var int User identifier */
     public $ID;
+
+    /** @var array User properties */
     public $data;
+
+    /** @var string User language code */
     public $language;
+
+    /** @var array User preferences */
     public $prefs;
 
-    /**
-     * Holds database connection.
-     *
-     * @var rcube_db
-     */
+
+    /** @var rcube_db Holds database connection */
     private $db;
 
-    /**
-     * Framework object.
-     *
-     * @var rcube
-     */
+    /** @var rcube Framework object */
     private $rc;
 
-    /**
-     * Internal identities cache
-     *
-     * @var array
-     */
-    private $identities = array();
+    /** @var array Internal identities cache */
+    private $identities = [];
 
-    /**
-     * Internal emails cache
-     *
-     * @var array
-     */
+    /** @var array Internal emails cache */
     private $emails;
 
 
     const SEARCH_ADDRESSBOOK = 1;
-    const SEARCH_MAIL = 2;
+    const SEARCH_MAIL        = 2;
+
 
     /**
      * Object constructor
@@ -77,32 +71,35 @@ class rcube_user
         if ($id && !$sql_arr) {
             $sql_result = $this->db->query(
                 "SELECT * FROM " . $this->db->table_name('users', true)
-                . " WHERE `user_id` = ?", $id);
+                . " WHERE `user_id` = ?", $id
+            );
+
             $sql_arr = $this->db->fetch_assoc($sql_result);
         }
 
         if (!empty($sql_arr)) {
-            $this->ID       = $sql_arr['user_id'];
+            $this->ID       = (int) $sql_arr['user_id'] ?? 0;
             $this->data     = $sql_arr;
-            $this->language = $sql_arr['language'];
+            $this->language = $sql_arr['language'] ?? null;
         }
     }
 
     /**
      * Build a user name string (as e-mail address)
      *
-     * @param  string $part Username part (empty or 'local' or 'domain', 'mail')
+     * @param string $part Username part (empty or 'local' or 'domain', 'mail')
+     *
      * @return string Full user name or its part
      */
     function get_username($part = null)
     {
-        if ($this->data['username']) {
+        if (!empty($this->data['username'])) {
             // return real name
             if (!$part) {
                 return $this->data['username'];
             }
 
-            list($local, $domain) = explode('@', $this->data['username']);
+            list($local, $domain) = rcube_utils::explode('@', $this->data['username']);
 
             // at least we should always have the local part
             if ($part == 'local') {
@@ -117,13 +114,12 @@ class rcube_user
                 return $domain;
             }
 
-            if (!empty($domain))
+            if (!empty($domain)) {
                 return $local . '@' . $domain;
-            else
-                return $local;
-        }
+            }
 
-        return false;
+            return $local;
+        }
     }
 
     /**
@@ -137,10 +133,11 @@ class rcube_user
             return $this->prefs;
         }
 
-        $this->prefs = array();
+        $this->prefs = [];
 
-        if (!empty($this->language))
+        if (!empty($this->language)) {
             $this->prefs['language'] = $this->language;
+        }
 
         if ($this->ID) {
             // Preferences from session (write-master is unavailable)
@@ -158,7 +155,7 @@ class rcube_user
             }
 
             if ($this->data['preferences']) {
-                $this->prefs += (array)unserialize($this->data['preferences']);
+                $this->prefs += (array) unserialize($this->data['preferences']);
             }
         }
 
@@ -171,32 +168,46 @@ class rcube_user
      * @param array $a_user_prefs User prefs to save
      * @param bool  $no_session   Simplified language/preferences handling
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     function save_prefs($a_user_prefs, $no_session = false)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
-        $plugin = $this->rc->plugins->exec_hook('preferences_update', array(
-            'userid' => $this->ID, 'prefs' => $a_user_prefs, 'old' => (array)$this->get_prefs()));
+        $config       = $this->rc->config;
+        $transient    = $config->transient_options();
+        $a_user_prefs = array_diff_key($a_user_prefs, array_flip($transient));
+
+        if (empty($a_user_prefs)) {
+            return true;
+        }
+
+        $plugin = $this->rc->plugins->exec_hook('preferences_update', [
+                'userid' => $this->ID,
+                'prefs'  => $a_user_prefs,
+                'old'    => (array)$this->get_prefs()
+        ]);
 
         if (!empty($plugin['abort'])) {
-            return;
+            return false;
         }
 
         $a_user_prefs = $plugin['prefs'];
         $old_prefs    = $plugin['old'];
-        $config       = $this->rc->config;
+        $defaults     = $config->all();
 
         // merge (partial) prefs array with existing settings
         $this->prefs = $save_prefs = $a_user_prefs + $old_prefs;
         unset($save_prefs['language']);
 
         // don't save prefs with default values if they haven't been changed yet
+        // Warning: we use result of rcube_config::all() here instead of just get() (#5782)
         foreach ($a_user_prefs as $key => $value) {
-            if ($value === null || (!isset($old_prefs[$key]) && ($value == $config->get($key))))
+            if ($value === null || (!isset($old_prefs[$key]) && isset($defaults[$key]) && $value === $defaults[$key])) {
                 unset($save_prefs[$key]);
+            }
         }
 
         $save_prefs = serialize($save_prefs);
@@ -210,14 +221,15 @@ class rcube_user
             " WHERE `user_id` = ?",
             $save_prefs,
             $this->language,
-            $this->ID);
+            $this->ID
+        );
 
         // Update success
         if ($this->db->affected_rows() !== false) {
             $this->data['preferences'] = $save_prefs;
 
             if (!$no_session) {
-                $config->set_user_prefs($a_user_prefs);
+                $config->set_user_prefs($this->prefs);
 
                 if (isset($_SESSION['preferences'])) {
                     $this->rc->session->remove('preferences');
@@ -235,7 +247,7 @@ class rcube_user
         ) {
             $_SESSION['preferences'] = $save_prefs;
             $_SESSION['preferences_time'] = time();
-            $config->set_user_prefs($a_user_prefs);
+            $config->set_user_prefs($this->prefs);
             $this->data['preferences'] = $save_prefs;
         }
 
@@ -243,7 +255,7 @@ class rcube_user
     }
 
     /**
-     * Generate a unique hash to identify this user whith
+     * Generate a unique hash to identify this user with
      */
     function get_hash()
     {
@@ -251,9 +263,8 @@ class rcube_user
 
         // generate a random hash and store it in user prefs
         if (empty($prefs['client_hash'])) {
-            mt_srand((double)microtime() * 1000000);
-            $prefs['client_hash'] = md5($this->data['username'] . mt_rand() . $this->data['mail_host']);
-            $this->save_prefs(array('client_hash' => $prefs['client_hash']));
+            $prefs['client_hash'] = rcube_utils::random_bytes(16);
+            $this->save_prefs(['client_hash' => $prefs['client_hash']]);
         }
 
         return $prefs['client_hash'];
@@ -262,21 +273,22 @@ class rcube_user
     /**
      * Return a list of all user emails (from identities)
      *
-     * @param bool Return only default identity
+     * @param bool $default Return only default identity
      *
-     * @return array List of emails (identity_id, name, email)
+     * @return array List of emails (identity_id, name, email) or single email-data
      */
     function list_emails($default = false)
     {
         if ($this->emails === null) {
-            $this->emails = array();
+            $this->emails = [];
 
             $sql_result = $this->db->query(
                 "SELECT `identity_id`, `name`, `email`"
                 ." FROM " . $this->db->table_name('identities', true)
                 ." WHERE `user_id` = ? AND `del` <> 1"
                 ." ORDER BY `standard` DESC, `name` ASC, `email` ASC, `identity_id` ASC",
-                $this->ID);
+                $this->ID
+            );
 
             while ($sql_arr = $this->db->fetch_assoc($sql_result)) {
                 $this->emails[] = $sql_arr;
@@ -289,12 +301,14 @@ class rcube_user
     /**
      * Get default identity of this user
      *
-     * @param  int   $id Identity ID. If empty, the default identity is returned
+     * @param int $id Identity ID. If empty, the default identity is returned
+     *
      * @return array Hash array with all cols of the identity record
      */
     function get_identity($id = null)
     {
-        $id = (int)$id;
+        $id = (int) $id;
+
         // cache identities for better performance
         if (!array_key_exists($id, $this->identities)) {
             $result = $this->list_identities($id ? "AND `identity_id` = $id" : '');
@@ -314,14 +328,14 @@ class rcube_user
      */
     function list_identities($sql_add = '', $formatted = false)
     {
-        $result = array();
+        $result = [];
 
         $sql_result = $this->db->query(
-            "SELECT * FROM ".$this->db->table_name('identities', true).
-            " WHERE `del` <> 1 AND `user_id` = ?".
-            ($sql_add ? " ".$sql_add : "").
-            " ORDER BY `standard` DESC, `name` ASC, `email` ASC, `identity_id` ASC",
-            $this->ID);
+            "SELECT * FROM ".$this->db->table_name('identities', true)
+            . " WHERE `del` <> 1 AND `user_id` = ?" . ($sql_add ? " $sql_add" : "")
+            . " ORDER BY `standard` DESC, `name` ASC, `email` ASC, `identity_id` ASC",
+            $this->ID
+        );
 
         while ($sql_arr = $this->db->fetch_assoc($sql_result)) {
             if ($formatted) {
@@ -342,18 +356,20 @@ class rcube_user
     /**
      * Update a specific identity record
      *
-     * @param int    $iid  Identity ID
-     * @param array  $data Hash array with col->value pairs to save
-     * @return boolean True if saved successfully, false if nothing changed
+     * @param int   $iid  Identity ID
+     * @param array $data Hash array with col->value pairs to save
+     *
+     * @return bool True if saved successfully, false if nothing changed
      */
     function update_identity($iid, $data)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
-        $query_cols = $query_params = array();
+        $query_cols = $query_params = [];
 
-        foreach ((array)$data as $col => $value) {
+        foreach ((array) $data as $col => $value) {
             $query_cols[]   = $this->db->quote_identifier($col) . ' = ?';
             $query_params[] = $value;
         }
@@ -361,91 +377,98 @@ class rcube_user
         $query_params[] = $this->ID;
 
         $sql = "UPDATE ".$this->db->table_name('identities', true).
-            " SET `changed` = ".$this->db->now().", ".join(', ', $query_cols).
-            " WHERE `identity_id` = ?".
-                " AND `user_id` = ?".
-                " AND `del` <> 1";
+            " SET `changed` = ".$this->db->now() . ", " . implode(', ', $query_cols)
+            . " WHERE `identity_id` = ?"
+                . " AND `user_id` = ?"
+                . " AND `del` <> 1";
 
-        call_user_func_array(array($this->db, 'query'),
-            array_merge(array($sql), $query_params));
+        $this->db->query($sql, $query_params);
 
         // clear the cache
-        $this->identities = array();
+        $this->identities = [];
         $this->emails     = null;
 
-        return $this->db->affected_rows();
+        return $this->db->affected_rows() > 0;
     }
 
     /**
      * Create a new identity record linked with this user
      *
      * @param array $data Hash array with col->value pairs to save
-     * @return int  The inserted identity ID or false on error
+     *
+     * @return int|false The inserted identity ID or false on error
      */
     function insert_identity($data)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
         unset($data['user_id']);
 
-        $insert_cols = $insert_values = array();
-        foreach ((array)$data as $col => $value) {
+        $insert_cols   = [];
+        $insert_values = [];
+
+        foreach ((array) $data as $col => $value) {
             $insert_cols[]   = $this->db->quote_identifier($col);
             $insert_values[] = $value;
         }
+
         $insert_cols[]   = $this->db->quote_identifier('user_id');
         $insert_values[] = $this->ID;
 
-        $sql = "INSERT INTO ".$this->db->table_name('identities', true).
-            " (`changed`, ".join(', ', $insert_cols).")".
-            " VALUES (".$this->db->now().", ".join(', ', array_pad(array(), sizeof($insert_values), '?')).")";
+        $sql = "INSERT INTO " . $this->db->table_name('identities', true)
+            . " (`changed`, " . implode(', ', $insert_cols) . ")"
+            . " VALUES (" . $this->db->now() . ", " . implode(', ', array_pad([], count($insert_values), '?')) . ")";
 
-        call_user_func_array(array($this->db, 'query'),
-            array_merge(array($sql), $insert_values));
+        $insert = $this->db->query($sql, $insert_values);
 
         // clear the cache
-        $this->identities = array();
+        $this->identities = [];
         $this->emails     = null;
 
-        return $this->db->insert_id('identities');
+        return $this->db->affected_rows($insert) ? $this->db->insert_id('identities') : false;
     }
 
     /**
      * Mark the given identity as deleted
      *
-     * @param  int     $iid Identity ID
-     * @return boolean True if deleted successfully, false if nothing changed
+     * @param int $iid Identity ID
+     *
+     * @return bool True if deleted successfully, false if nothing changed
      */
     function delete_identity($iid)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
         $sql_result = $this->db->query(
-            "SELECT count(*) AS ident_count FROM ".$this->db->table_name('identities', true).
-            " WHERE `user_id` = ? AND `del` <> 1",
-            $this->ID);
+            "SELECT count(*) AS ident_count FROM " . $this->db->table_name('identities', true)
+            . " WHERE `user_id` = ? AND `del` <> 1",
+            $this->ID
+        );
 
         $sql_arr = $this->db->fetch_assoc($sql_result);
 
         // we'll not delete last identity
-        if ($sql_arr['ident_count'] <= 1)
-            return -1;
+        if ($sql_arr['ident_count'] <= 1) {
+            return false;
+        }
 
         $this->db->query(
-            "UPDATE ".$this->db->table_name('identities', true).
-            " SET `del` = 1, `changed` = ".$this->db->now().
-            " WHERE `user_id` = ?".
-                " AND `identity_id` = ?",
+            "UPDATE " . $this->db->table_name('identities', true)
+            . " SET `del` = 1, `changed` = " . $this->db->now()
+            . " WHERE `user_id` = ? AND `identity_id` = ?",
             $this->ID,
-            $iid);
+            $iid
+        );
 
         // clear the cache
-        $this->identities = array();
+        $this->identities = [];
         $this->emails     = null;
 
-        return $this->db->affected_rows();
+        return $this->db->affected_rows() > 0;
     }
 
     /**
@@ -457,13 +480,22 @@ class rcube_user
     {
         if ($this->ID && $iid) {
             $this->db->query(
-                "UPDATE ".$this->db->table_name('identities', true).
-                " SET `standard` = '0'".
-                " WHERE `user_id` = ? AND `identity_id` <> ?",
+                "UPDATE " . $this->db->table_name('identities', true)
+                . " SET `standard` = '0'"
+                . " WHERE `user_id` = ? AND `identity_id` <> ?",
                 $this->ID,
-                $iid);
+                $iid
+            );
 
-            unset($this->identities[0]);
+            $this->db->query(
+                "UPDATE " . $this->db->table_name('identities', true)
+                . " SET `standard` = '1'"
+                . " WHERE `user_id` = ? AND `identity_id` = ?",
+                $this->ID,
+                $iid
+            );
+
+            $this->identities = [];
         }
     }
 
@@ -474,19 +506,73 @@ class rcube_user
     {
         if ($this->ID) {
             $this->db->query(
-                "UPDATE ".$this->db->table_name('users', true).
-                " SET `last_login` = ".$this->db->now().
-                " WHERE `user_id` = ?",
-                $this->ID);
+                "UPDATE " . $this->db->table_name('users', true)
+                . " SET `last_login` = " . $this->db->now()
+                . " WHERE `user_id` = ?",
+                $this->ID
+            );
         }
     }
 
+    /**
+     * Update user's failed_login timestamp and counter
+     */
+    function failed_login()
+    {
+        if ($this->ID && $this->rc->config->get('login_rate_limit', 3)) {
+            $counter = 0;
+
+            if (empty($this->data['failed_login'])) {
+                $failed_login = new DateTime('now');
+                $counter      = 1;
+            }
+            else {
+                $failed_login = new DateTime($this->data['failed_login']);
+                $threshold    = new DateTime('- 60 seconds');
+
+                if ($failed_login < $threshold) {
+                    $failed_login = new DateTime('now');
+                    $counter      = 1;
+                }
+            }
+
+            $this->db->query(
+                "UPDATE " . $this->db->table_name('users', true)
+                    . " SET `failed_login` = ?"
+                    . ", `failed_login_counter` = " . ($counter ?: "`failed_login_counter` + 1")
+                . " WHERE `user_id` = ?",
+                $failed_login, $this->ID
+            );
+        }
+    }
+
+    /**
+     * Checks if the account is locked, e.g. as a result of brute-force prevention
+     */
+    function is_locked()
+    {
+        if (empty($this->data['failed_login'])) {
+            return false;
+        }
+
+        if ($rate = (int) $this->rc->config->get('login_rate_limit', 3)) {
+            $last_failed = new DateTime($this->data['failed_login']);
+            $threshold   = new DateTime('- 60 seconds');
+
+            if ($last_failed > $threshold && $this->data['failed_login_counter'] >= $rate) {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Clear the saved object state
      */
     function reset()
     {
-        $this->ID = null;
+        $this->ID   = null;
         $this->data = null;
     }
 
@@ -495,6 +581,7 @@ class rcube_user
      *
      * @param string $user IMAP user name
      * @param string $host IMAP host name
+     *
      * @return rcube_user New user instance
      */
     static function query($user, $host)
@@ -511,18 +598,19 @@ class rcube_user
         // username not found, try aliases from identities
         if (empty($sql_arr) && $config->get('user_aliases') && strpos($user, '@')) {
             $sql_result = $dbh->limitquery("SELECT u.*"
-                ." FROM " . $dbh->table_name('users', true) . " u"
-                ." JOIN " . $dbh->table_name('identities', true) . " i ON (i.`user_id` = u.`user_id`)"
-                ." WHERE `email` = ? AND `del` <> 1", 0, 1, $user);
+                . " FROM " . $dbh->table_name('users', true) . " u"
+                . " JOIN " . $dbh->table_name('identities', true) . " i ON (i.`user_id` = u.`user_id`)"
+                . " WHERE `email` = ? AND `del` <> 1",
+                0, 1, $user
+            );
 
             $sql_arr = $dbh->fetch_assoc($sql_result);
         }
 
         // user already registered -> overwrite username
-        if ($sql_arr)
+        if ($sql_arr) {
             return new rcube_user($sql_arr['user_id'], $sql_arr);
-        else
-            return false;
+        }
     }
 
     /**
@@ -530,7 +618,8 @@ class rcube_user
      *
      * @param string $user IMAP user name
      * @param string $host IMAP host
-     * @return rcube_user New user instance
+     *
+     * @return rcube_user|null New user instance on success, Null on error/abort
      */
     static function create($user, $host)
     {
@@ -544,36 +633,41 @@ class rcube_user
             $user_email = is_array($email_list[0]) ? $email_list[0]['email'] : $email_list[0];
         }
 
-        $data = $rcube->plugins->exec_hook('user_create', array(
-            'host'       => $host,
-            'user'       => $user,
-            'user_name'  => $user_name,
-            'user_email' => $user_email,
-            'email_list' => $email_list,
-            'language'   =>  $_SESSION['language'],
-        ));
+        $data = $rcube->plugins->exec_hook('user_create', [
+                'host'        => $host,
+                'user'        => $user,
+                'user_name'   => $user_name,
+                'user_email'  => $user_email,
+                'email_list'  => $email_list,
+                'language'    => $_SESSION['language'] ?? null,
+                'preferences' => [],
+        ]);
 
         // plugin aborted this operation
         if ($data['abort']) {
-            return false;
+            return;
         }
 
-        $dbh->query(
-            "INSERT INTO ".$dbh->table_name('users', true).
-            " (`created`, `last_login`, `username`, `mail_host`, `language`)".
-            " VALUES (".$dbh->now().", ".$dbh->now().", ?, ?, ?)",
+        $insert = $dbh->query(
+            "INSERT INTO " . $dbh->table_name('users', true)
+            . " (`created`, `last_login`, `username`, `mail_host`, `language`, `preferences`)"
+            . " VALUES (" . $dbh->now() . ", " . $dbh->now() . ", ?, ?, ?, ?)",
             $data['user'],
             $data['host'],
-            $data['language']);
+            $data['language'],
+            serialize($data['preferences'])
+        );
 
-        if ($user_id = $dbh->insert_id('users')) {
+        if ($dbh->affected_rows($insert) && ($user_id = $dbh->insert_id('users'))) {
             // create rcube_user instance to make plugin hooks work
-            $user_instance = new rcube_user($user_id, array(
-                'user_id'   => $user_id,
-                'username'  => $data['user'],
-                'mail_host' => $data['host'],
-                'language'  => $data['language'],
-            ));
+            $user_instance = new rcube_user($user_id, [
+                    'user_id'     => $user_id,
+                    'username'    => $data['user'],
+                    'mail_host'   => $data['host'],
+                    'language'    => $data['language'],
+                    'preferences' => serialize($data['preferences']),
+            ]);
+
             $rcube->user = $user_instance;
             $mail_domain = $rcube->config->mail_domain($data['host']);
             $user_name   = $data['user_name'];
@@ -588,7 +682,7 @@ class rcube_user
             }
             // identities_level check
             else if (count($email_list) > 1 && $rcube->config->get('identities_level', 0) > 1) {
-                $email_list = array($email_list[0]);
+                $email_list = [$email_list[0]];
             }
 
             if (empty($user_name)) {
@@ -598,7 +692,7 @@ class rcube_user
             // create new identities records
             $standard = 1;
             foreach ($email_list as $row) {
-                $record = array();
+                $record = [];
 
                 if (is_array($row)) {
                     if (empty($row['email'])) {
@@ -618,37 +712,38 @@ class rcube_user
                 $record['standard'] = $standard;
 
                 $plugin = $rcube->plugins->exec_hook('identity_create',
-                    array('login' => true, 'record' => $record));
+                    ['login' => true, 'record' => $record]);
 
                 if (!$plugin['abort'] && $plugin['record']['email']) {
                     $rcube->user->insert_identity($plugin['record']);
                 }
+
                 $standard = 0;
             }
         }
         else {
-            rcube::raise_error(array(
-                'code' => 500,
-                'type' => 'php',
-                'line' => __LINE__,
-                'file' => __FILE__,
-                'message' => "Failed to create new user"), true, false);
+            rcube::raise_error([
+                    'code' => 500, 'line' => __LINE__, 'file' => __FILE__,
+                    'message' => "Failed to create new user"
+                ],
+                true, false
+            );
         }
 
-        return $user_id ? $user_instance : false;
+        return !empty($user_instance) ? $user_instance : null;
     }
 
     /**
      * Resolve username using a virtuser plugins
      *
      * @param string $email E-mail address to resolve
+     *
      * @return string Resolved IMAP username
      */
     static function email2user($email)
     {
-        $rcube = rcube::get_instance();
-        $plugin = $rcube->plugins->exec_hook('email2user',
-            array('email' => $email, 'user' => NULL));
+        $rcube  = rcube::get_instance();
+        $plugin = $rcube->plugins->exec_hook('email2user', ['email' => $email, 'user' => null]);
 
         return $plugin['user'];
     }
@@ -656,47 +751,51 @@ class rcube_user
     /**
      * Resolve e-mail address from virtuser plugins
      *
-     * @param string $user User name
-     * @param boolean $first If true returns first found entry
-     * @param boolean $extended If true returns email as array (email and name for identity)
+     * @param string $user     User name
+     * @param bool   $first    If true returns first found entry
+     * @param bool   $extended If true returns email as array (email and name for identity)
+     *
      * @return mixed Resolved e-mail address string or array of strings
      */
-    static function user2email($user, $first=true, $extended=false)
+    static function user2email($user, $first = true, $extended = false)
     {
-        $rcube = rcube::get_instance();
-        $plugin = $rcube->plugins->exec_hook('user2email',
-            array('email' => NULL, 'user' => $user,
-                'first' => $first, 'extended' => $extended));
+        $rcube  = rcube::get_instance();
+        $plugin = $rcube->plugins->exec_hook('user2email', [
+                'email'    => null,
+                'user'     => $user,
+                'first'    => $first,
+                'extended' => $extended
+        ]);
 
-        return empty($plugin['email']) ? NULL : $plugin['email'];
+        return empty($plugin['email']) ? null : $plugin['email'];
     }
 
     /**
      * Return a list of saved searches linked with this user
      *
-     * @param int  $type  Search type
+     * @param int $type Search type
      *
      * @return array List of saved searches indexed by search ID
      */
     function list_searches($type)
     {
-        $plugin = $this->rc->plugins->exec_hook('saved_search_list', array('type' => $type));
+        $plugin = $this->rc->plugins->exec_hook('saved_search_list', ['type' => $type]);
 
         if ($plugin['abort']) {
             return (array) $plugin['result'];
         }
 
-        $result = array();
+        $result = [];
 
         $sql_result = $this->db->query(
             "SELECT `search_id` AS id, `name`"
-            ." FROM ".$this->db->table_name('searches', true)
-            ." WHERE `user_id` = ? AND `type` = ?"
-            ." ORDER BY `name`",
-            (int) $this->ID, (int) $type);
+            . " FROM " . $this->db->table_name('searches', true)
+            . " WHERE `user_id` = ? AND `type` = ?"
+            . " ORDER BY `name`",
+            (int) $this->ID, (int) $type
+        );
 
         while ($sql_arr = $this->db->fetch_assoc($sql_result)) {
-            $sql_arr['data'] = unserialize($sql_arr['data']);
             $result[$sql_arr['id']] = $sql_arr;
         }
 
@@ -706,56 +805,57 @@ class rcube_user
     /**
      * Return saved search data.
      *
-     * @param int  $id  Row identifier
+     * @param int $id Row identifier
      *
      * @return array Data
      */
     function get_search($id)
     {
-        $plugin = $this->rc->plugins->exec_hook('saved_search_get', array('id' => $id));
+        $plugin = $this->rc->plugins->exec_hook('saved_search_get', ['id' => $id]);
 
         if ($plugin['abort']) {
-            return $plugin['result'];
+            return (array) $plugin['result'];
         }
 
         $sql_result = $this->db->query(
             "SELECT `name`, `data`, `type`"
             . " FROM ".$this->db->table_name('searches', true)
-            . " WHERE `user_id` = ?"
-                ." AND `search_id` = ?",
-            (int) $this->ID, (int) $id);
+            . " WHERE `user_id` = ? AND `search_id` = ?",
+            (int) $this->ID, (int) $id
+        );
 
         while ($sql_arr = $this->db->fetch_assoc($sql_result)) {
-            return array(
+            return [
                 'id'   => $id,
                 'name' => $sql_arr['name'],
                 'type' => $sql_arr['type'],
                 'data' => unserialize($sql_arr['data']),
-            );
+            ];
         }
 
-        return null;
+        return [];
     }
 
     /**
      * Deletes given saved search record
      *
-     * @param  int  $sid  Search ID
+     * @param int $sid Search ID
      *
-     * @return boolean True if deleted successfully, false if nothing changed
+     * @return bool True if deleted successfully, false if nothing changed
      */
     function delete_search($sid)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
         $this->db->query(
-            "DELETE FROM ".$this->db->table_name('searches', true)
-            ." WHERE `user_id` = ?"
-                ." AND `search_id` = ?",
-            (int) $this->ID, $sid);
+            "DELETE FROM " . $this->db->table_name('searches', true)
+            ." WHERE `user_id` = ? AND `search_id` = ?",
+            (int) $this->ID, $sid
+        );
 
-        return $this->db->affected_rows();
+        return $this->db->affected_rows() > 0;
     }
 
     /**
@@ -763,12 +863,13 @@ class rcube_user
      *
      * @param array $data Hash array with col->value pairs to save
      *
-     * @return int  The inserted search ID or false on error
+     * @return int The inserted search ID or false on error
      */
     function insert_search($data)
     {
-        if (!$this->ID)
+        if (!$this->ID) {
             return false;
+        }
 
         $insert_cols[]   = 'user_id';
         $insert_values[] = (int) $this->ID;
@@ -779,13 +880,164 @@ class rcube_user
         $insert_cols[]   = $this->db->quote_identifier('data');
         $insert_values[] = serialize($data['data']);
 
-        $sql = "INSERT INTO ".$this->db->table_name('searches', true)
-            ." (".join(', ', $insert_cols).")"
-            ." VALUES (".join(', ', array_pad(array(), sizeof($insert_values), '?')).")";
+        $sql = "INSERT INTO " . $this->db->table_name('searches', true)
+            . " (" . implode(', ', $insert_cols) . ")"
+            . " VALUES (" . implode(', ', array_pad([], count($insert_values), '?')) . ")";
 
-        call_user_func_array(array($this->db, 'query'),
-            array_merge(array($sql), $insert_values));
+        $insert = $this->db->query($sql, $insert_values);
 
-        return $this->db->insert_id('searches');
+        return $this->db->affected_rows($insert) ? $this->db->insert_id('searches') : false;
+    }
+
+    /**
+     * Get a saved response of this user.
+     *
+     * @param int $id Response ID
+     *
+     * @return array|null Hash array with all cols of the response record, NULL if not found
+     */
+    function get_response($id)
+    {
+        $sql_result = $this->db->query(
+            "SELECT * FROM " . $this->db->table_name('responses', true)
+                . " WHERE `user_id` = ? AND `response_id` = ? AND `del` = 0",
+            $this->ID, $id
+        );
+
+        if ($sql_arr = $this->db->fetch_assoc($sql_result)) {
+            $sql_arr['id']      = $sql_arr['response_id'];
+            $sql_arr['is_html'] = !empty($sql_arr['is_html']);
+
+            unset($sql_arr['response_id']);
+
+            return $sql_arr;
+        }
+    }
+
+    /**
+     * Return a list of all responses of this user.
+     *
+     * @return array List of responses (id, name)
+     */
+    function list_responses()
+    {
+        // Migrate the old responses existing in user preferences
+        if (!empty($this->prefs['compose_responses'])) {
+            foreach ($this->prefs['compose_responses'] as $response) {
+                $this->insert_response([
+                        'name' => $response['name'],
+                        'data' => $response['text'],
+                ]);
+            }
+
+            $this->save_prefs(['compose_responses' => null]);
+        }
+
+        $sql_result = $this->db->query(
+            "SELECT `response_id`, `name` FROM " . $this->db->table_name('responses', true)
+                . " WHERE `user_id` = ? AND `del` = 0"
+                . " ORDER BY `name`",
+            $this->ID
+        );
+
+        $result = [];
+
+        while ($sql_arr = $this->db->fetch_assoc($sql_result)) {
+            $result[] = [
+                'id'   => $sql_arr['response_id'],
+                'name' => $sql_arr['name'],
+            ];
+        }
+
+        return $result;
+    }
+
+    /**
+     * Update a specific response record.
+     *
+     * @param int   $id   Response ID
+     * @param array $data Hash array with col->value pairs to save
+     *
+     * @return bool True if saved successfully, false if nothing changed
+     */
+    function update_response($id, $data)
+    {
+        if (!$this->ID) {
+            return false;
+        }
+
+        $query_cols = $query_params = [];
+
+        foreach (['name', 'data', 'is_html'] as $col) {
+            if (isset($data[$col])) {
+                $query_cols[]   = $this->db->quote_identifier($col) . ' = ?';
+                $query_params[] = $col == 'is_html' ? intval($data[$col]) : $data[$col];
+            }
+        }
+        $query_params[] = $id;
+        $query_params[] = $this->ID;
+
+        $sql = "UPDATE " . $this->db->table_name('responses', true)
+            . " SET `changed` = " . $this->db->now() . ", " . implode(', ', $query_cols)
+            . " WHERE `response_id` = ? AND `user_id` = ? AND `del` = 0";
+
+        $this->db->query($sql, $query_params);
+
+        return $this->db->affected_rows() > 0;
+    }
+
+    /**
+     * Create a new response record for the user.
+     *
+     * @param array $data Hash array with col->value pairs to save
+     *
+     * @return int|false The inserted response ID or false on error
+     */
+    function insert_response($data)
+    {
+        if (!$this->ID) {
+            return false;
+        }
+
+        $query_cols   = [$this->db->quote_identifier('user_id')];
+        $query_params = [$this->ID];
+
+        foreach (['name', 'data', 'is_html'] as $col) {
+            if (isset($data[$col])) {
+                $query_cols[]   = $this->db->quote_identifier($col);
+                $query_params[] = $col == 'is_html' ? intval($data[$col]) : $data[$col];
+            }
+        }
+
+        $sql = "INSERT INTO " . $this->db->table_name('responses', true)
+            . " (`changed`, " . implode(', ', $query_cols) . ")"
+            . " VALUES (" . $this->db->now() . ", " . trim(str_repeat('?, ', count($query_cols)), ', ') . ")";
+
+        $insert = $this->db->query($sql, $query_params);
+
+        return $this->db->affected_rows($insert) ? $this->db->insert_id('responses') : false;
+    }
+
+    /**
+     * Delete the given response record
+     *
+     * @param int $id Response ID
+     *
+     * @return bool True if deleted successfully, false otherwise
+     */
+    function delete_response($id)
+    {
+        if (!$this->ID) {
+            return false;
+        }
+
+        $this->db->query(
+            "UPDATE " . $this->db->table_name('responses', true)
+                . " SET `del` = 1 WHERE `user_id` = ? AND `response_id` = ?",
+            $this->ID,
+            $id
+        );
+
+        return $this->db->affected_rows() > 0;
     }
 }
